@@ -252,9 +252,10 @@
 	(if entry
 	    (setcdr entry
 		    (concat (cdr entry)
-			    (substring string (match-end 0))))
+			    (substring string (match-end 0)) " "))
 	  (setq riece-353-message-alist
-		(cons (cons channel (substring string (match-end 0)))
+		(cons (cons channel
+			    (concat (substring string (match-end 0)) " "))
 		      riece-353-message-alist))))))
 
 (defun riece-handle-322-message (prefix number name decoded)
@@ -438,7 +439,7 @@
 	    (setq riece-353-message-alist
 		  (delq entry riece-353-message-alist)))
 	(while (string-match
-		(concat "\\([@+]\\)?\\([^ ]+\\) *")
+		(concat "\\([@+]\\)?\\([^ ]+\\) +")
 		string start)
 	  (put-text-property (match-beginning 2) (match-end 2)
 			     'riece-identity
