@@ -19,8 +19,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Code:
 
@@ -69,7 +69,7 @@
 		      (quit
 		       (if (equal server-name "")
 			   (message "Password: Quit")
-			 (message "Password for %s: " server-name))
+			 (message "Password for %s: Quit" server-name))
 		       'quit)))
 	    (setq riece-reconnect-with-password nil)))
       (if (eq password 'quit)
@@ -79,8 +79,9 @@
 				       (format "PASS %s\r\n" password)))
 	(riece-process-send-string process
 				   (format "USER %s * * :%s\r\n"
-					   (user-real-login-name)
 					   (or username
+					       (user-real-login-name))
+					   (or realname
 					       "No information given")))
 	(riece-process-send-string process (format "NICK %s\r\n" nickname))
 	(with-current-buffer (process-buffer process)
