@@ -250,6 +250,8 @@ If optional argument SAFE is nil, overwrite previous definitions."
 (defun riece-save-variables-files ()
   "Save current settings to `riece-saved-variables-file'."
   (with-temp-file riece-saved-variables-file
+    (insert ";;; This file is generated automatically by " riece-version ".\n"
+	    ";;; Do not edit this file!\n\n")
     (let ((print-quoted t)
 	  (print-readably t)
 	  print-escape-multibyte
